// HiSlipRandomTest.cpp : Defines the entry point for the console application.
//


#include "stdafx.h"

using namespace std;


static unsigned s_uErrorCount = 0;


int main(int argc, char* argv[])
{
  if ( argc >= 2 )
  {
    StartTest(argv[1]);
  }
  else
  {
    StartTest("TCPIP::localhost::hislip0");
  }

  if ( s_uErrorCount > 0 )
  {
    cout << "\n!!Test finished with " << s_uErrorCount << " errors!!\n";
	return 1;
  }
  else
  {
    cout << "\nTest finished with no errors\n";
  }

	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////

bool CompareResult( ViStatus expected, ViStatus status, const char* sTest, unsigned uLine, char *sFile )
{
  if ( status != expected )
  {
    cout << endl << endl;
    cout << sFile << "(" << uLine << "):" << endl;
    cout << "Test: " << sTest << endl;
    cout << "Unexpected result: (expected) " << GetViStatusString( expected ) << " != (result) " 
         << GetViStatusString( status ) << endl;
    _getch();
    s_uErrorCount++;
  }
  return (status  == expected);
}

bool TestLockFailed( ViStatus status, const char* sTest, unsigned uLine, char *sFile )
{
  // VISAs don't have identical result codes.
  if ( status != VI_ERROR_TMO && status != VI_ERROR_RSRC_LOCKED )
  {
    cout << endl << endl;
    cout << sFile << "(" << uLine << "):" << endl;
    cout << "Test: " << sTest << endl;
    cout << "Unexpected lock failure: " << GetViStatusString( status ) << endl;
    _getch();
    s_uErrorCount++;
    return false;
  }
  return true;
}

bool Check( bool bIsTrue,  const char* sTest, unsigned uLine, char *sFile )
{
  if ( !bIsTrue )
  {
    cout << endl << endl;
    cout << "Check: " << sTest << endl;
    cout << sFile << "(" << uLine << "): Condition failed" << endl;
    _getch();
    s_uErrorCount++;
  }
  return bIsTrue;
}


const char* GetViStatusString( ViStatus status ) 
{
  #define CASE_CODE(c) case c: sError = #c; break; 

  const char * sError = "Error Code not defined";

  switch ( status )
  {
    CASE_CODE(VI_SUCCESS)
    CASE_CODE(VI_SUCCESS_EVENT_EN)
    CASE_CODE(VI_SUCCESS_EVENT_DIS)
    CASE_CODE(VI_SUCCESS_QUEUE_EMPTY)
    CASE_CODE(VI_SUCCESS_TERM_CHAR)
    CASE_CODE(VI_SUCCESS_MAX_CNT)
    CASE_CODE(VI_SUCCESS_DEV_NPRESENT)
    CASE_CODE(VI_SUCCESS_TRIG_MAPPED)
    CASE_CODE(VI_SUCCESS_QUEUE_NEMPTY)
    CASE_CODE(VI_SUCCESS_NCHAIN)
    CASE_CODE(VI_SUCCESS_NESTED_SHARED)
    CASE_CODE(VI_SUCCESS_NESTED_EXCLUSIVE)
    CASE_CODE(VI_SUCCESS_SYNC)
    CASE_CODE(VI_WARN_CONFIG_NLOADED)
    CASE_CODE(VI_WARN_NULL_OBJECT)
    CASE_CODE(VI_WARN_NSUP_ATTR_STATE)
    CASE_CODE(VI_WARN_UNKNOWN_STATUS)
    CASE_CODE(VI_WARN_NSUP_BUF)
    CASE_CODE(VI_WARN_EXT_FUNC_NIMPL)
    CASE_CODE(VI_ERROR_SYSTEM_ERROR)
    CASE_CODE(VI_ERROR_INV_OBJECT)
    CASE_CODE(VI_ERROR_RSRC_LOCKED)
    CASE_CODE(VI_ERROR_INV_EXPR)
    CASE_CODE(VI_ERROR_RSRC_NFOUND)
    CASE_CODE(VI_ERROR_INV_RSRC_NAME)
    CASE_CODE(VI_ERROR_INV_ACC_MODE)
    CASE_CODE(VI_ERROR_TMO)
    CASE_CODE(VI_ERROR_CLOSING_FAILED)
    CASE_CODE(VI_ERROR_INV_DEGREE)
    CASE_CODE(VI_ERROR_INV_JOB_ID)
    CASE_CODE(VI_ERROR_NSUP_ATTR)
    CASE_CODE(VI_ERROR_NSUP_ATTR_STATE)
    CASE_CODE(VI_ERROR_ATTR_READONLY)
    CASE_CODE(VI_ERROR_INV_LOCK_TYPE)
    CASE_CODE(VI_ERROR_INV_ACCESS_KEY)
    CASE_CODE(VI_ERROR_INV_EVENT)
    CASE_CODE(VI_ERROR_INV_MECH)
    CASE_CODE(VI_ERROR_HNDLR_NINSTALLED)
    CASE_CODE(VI_ERROR_INV_HNDLR_REF)
    CASE_CODE(VI_ERROR_INV_CONTEXT)
    //CASE_CODE(VI_ERROR_QUEUE_OVERFLOW)
    CASE_CODE(VI_ERROR_NENABLED)
    CASE_CODE(VI_ERROR_ABORT)
    CASE_CODE(VI_ERROR_RAW_WR_PROT_VIOL)
    CASE_CODE(VI_ERROR_RAW_RD_PROT_VIOL)
    CASE_CODE(VI_ERROR_OUTP_PROT_VIOL)
    CASE_CODE(VI_ERROR_INP_PROT_VIOL)
    CASE_CODE(VI_ERROR_BERR)
    CASE_CODE(VI_ERROR_IN_PROGRESS)
    CASE_CODE(VI_ERROR_INV_SETUP)
    CASE_CODE(VI_ERROR_QUEUE_ERROR)
    CASE_CODE(VI_ERROR_ALLOC)
    CASE_CODE(VI_ERROR_INV_MASK)
    CASE_CODE(VI_ERROR_IO)
    CASE_CODE(VI_ERROR_INV_FMT)
    CASE_CODE(VI_ERROR_NSUP_FMT)
    CASE_CODE(VI_ERROR_LINE_IN_USE)
    CASE_CODE(VI_ERROR_NSUP_MODE)
    CASE_CODE(VI_ERROR_SRQ_NOCCURRED)
    CASE_CODE(VI_ERROR_INV_SPACE)
    CASE_CODE(VI_ERROR_INV_OFFSET)
    CASE_CODE(VI_ERROR_INV_WIDTH)
    CASE_CODE(VI_ERROR_NSUP_OFFSET)
    CASE_CODE(VI_ERROR_NSUP_VAR_WIDTH)
    CASE_CODE(VI_ERROR_WINDOW_NMAPPED)
    CASE_CODE(VI_ERROR_RESP_PENDING)
    CASE_CODE(VI_ERROR_NLISTENERS)
    CASE_CODE(VI_ERROR_NCIC)
    CASE_CODE(VI_ERROR_NSYS_CNTLR)
    CASE_CODE(VI_ERROR_NSUP_OPER)
    CASE_CODE(VI_ERROR_INTR_PENDING)
    CASE_CODE(VI_ERROR_ASRL_PARITY)
    CASE_CODE(VI_ERROR_ASRL_FRAMING)
    CASE_CODE(VI_ERROR_ASRL_OVERRUN)
    CASE_CODE(VI_ERROR_TRIG_NMAPPED)
    CASE_CODE(VI_ERROR_NSUP_ALIGN_OFFSET)
    CASE_CODE(VI_ERROR_USER_BUF)
    CASE_CODE(VI_ERROR_RSRC_BUSY)
    CASE_CODE(VI_ERROR_NSUP_WIDTH)
    CASE_CODE(VI_ERROR_INV_PARAMETER)
    CASE_CODE(VI_ERROR_INV_PROT)
    CASE_CODE(VI_ERROR_INV_SIZE)
    CASE_CODE(VI_ERROR_WINDOW_MAPPED)
    CASE_CODE(VI_ERROR_NIMPL_OPER)
    CASE_CODE(VI_ERROR_INV_LENGTH)
    CASE_CODE(VI_ERROR_INV_MODE)
    CASE_CODE(VI_ERROR_SESN_NLOCKED)
    CASE_CODE(VI_ERROR_MEM_NSHARED)
    CASE_CODE(VI_ERROR_LIBRARY_NFOUND)
    CASE_CODE(VI_ERROR_NSUP_INTR)
    CASE_CODE(VI_ERROR_INV_LINE)
    CASE_CODE(VI_ERROR_FILE_ACCESS)
    CASE_CODE(VI_ERROR_FILE_IO)
    CASE_CODE(VI_ERROR_NSUP_LINE)
    CASE_CODE(VI_ERROR_NSUP_MECH)
    CASE_CODE(VI_ERROR_INTF_NUM_NCONFIG)
    CASE_CODE(VI_ERROR_CONN_LOST)
  }

  return sError;

  #undef CASE_CODE
}
