#include "stdafx.h"
#include <atltime.h>
#include <atlstr.h>

using namespace std;

#define AGILENT 0

static ViUInt16 s_manfId = 'RS'; // Do not change. The right value is set below.
static bool s_bSrqEvent = false;

/* FUNCTION *******************************************************************/
/**
@note Send
*******************************************************************************/
static ViStatus Send( ViSession vi, char *sCommand, bool bEnd )
{
  ViUInt32 len, count;
  len = static_cast<ViUInt32>(strlen(sCommand));
  ViStatus status;

#if 0
  status = viSetAttribute( vi, VI_ATTR_SEND_END_EN, bEnd );
  if ( status < VI_SUCCESS ) return status;
#endif  

  status = viWrite( vi, ViBuf(sCommand), len, &count );
  return status;
}

/* FUNCTION *******************************************************************/
/**
@note ReadInteger
*******************************************************************************/
static ViStatus ReadInteger(int lineNo, ViSession vi )
{
  ViStatus status;
  ViUInt32 count;
  char buf[256];

  status = viRead( vi, (ViPBuf)buf, sizeof(buf)-1, &count );
  if ( !CompareResult( VI_SUCCESS, status, "ReadInteger", lineNo, __FILE__ ) )
  {
    return status;
  }
  buf[count] = '\0';

  // TODO: check number
  return status;
}

/* FUNCTION *******************************************************************/
/**
@note CheckResponse
*******************************************************************************/
static ViStatus CheckResponse(int lineNo, ViSession vi, char *sResponse )
{
  ViStatus status;
  ViUInt32 count;
  char buf[256];

  status = viRead( vi, (ViPBuf)buf, sizeof(buf)-1, &count );
  if ( !CompareResult( VI_SUCCESS, status, "CheckResponse", lineNo, __FILE__ ) )
  {
    return status;
  }
  buf[count] = '\0';

  if ( !CHECK( _stricmp( buf, sResponse ) == 0 ) )
  {
    cout << "  Expected '" << sResponse << "'\n  Read '" << buf << "'" << endl;
    cout << "  CheckResponse called from line " << lineNo << endl;
  }
  return status;
}

/* FUNCTION *******************************************************************/
/**
@note CheckNoError
*******************************************************************************/
static void CheckNoError( int lineNo, ViSession vi )
{
  if ( s_manfId == 'AG' )
  {
    if ( VI_SUCCESS != CheckResponse( lineNo, vi, "+0,\"No error\"\n" ) )
      cout << "  CheckNoError called from line " << lineNo << endl;
  }
  else
  {
    if ( VI_SUCCESS != CheckResponse( lineNo, vi, "0,\"No error\"\n" ) )
      cout << "  CheckNoError called from line " << lineNo << endl;
  }
}

/* FUNCTION *******************************************************************/
/**
@note CheckNoError
*******************************************************************************/
static void SleepOneSecond( ViSession vi )
{
  if ( s_manfId == 'AG' )
  {
    TR( Send( vi, "SLEEP 1", true ) );
  }
  else if ( s_manfId == 'GK' ) // Tiny HislipServer from website
  {
    TR( Send( vi, "SLEEP 1000", true ) );
  }
  else if ( s_manfId == 'RS' )
  {
    TR( Send( vi, "test:scpi:internal:sync 1000", true ) );
  }
  else
  {
    cout << "Unknown server. Please enter a valid ManfId in line: " << __LINE__ << endl;
  }
}

/* FUNCTION *******************************************************************/
/**
@note RandomSleep
*******************************************************************************/
static void RandomSleep()
{
  if ( rand() % 2 ) 
    Sleep( 1 + rand() % 200 );
}


/* FUNCTION *******************************************************************/
/**
@note CheckQueryError
*******************************************************************************/
static void CheckQueryError( int lineNo, ViSession vi )
{
  if ( VI_SUCCESS != CheckResponse( lineNo, vi, "-410,\"Query interrupted\"\n" ) )
    cout << "  CheckQueryError called from line " << lineNo << endl;
}

/* FUNCTION *******************************************************************/
/**
@note TestMAV
*******************************************************************************/
static void TestMAV( ViSession vi, bool bIsSet, int lineNo )
{
  ViUInt16 stb;
  bool result;

  TR(viReadSTB( vi, &stb ));
  if ( bIsSet )
  {
    result = CHECK( ((stb & 0x10) > 0) );
  }
  else
  {
    result = CHECK( ((stb & 0x10) == 0) );
  }
  if (!result)
  {
    cout << "  STB = " << stb << endl;
    cout << "  TestMAV called from line " << lineNo << endl;
  }
}

/* FUNCTION *******************************************************************/
/**
@note CheckLockState
*******************************************************************************/
static void CheckLockState( int lineNo, ViSession vi, ViAttrState state )
{
  ViAttrState lockState = 0;
  TR( viGetAttribute( vi, VI_ATTR_RSRC_LOCK_STATE, &lockState ));
  if (! Check( lockState == state, "CheckLockState", lineNo, __FILE__ ) )
  {
    cout << "Actual lockState " << lockState << " != expected state " << state << endl;
    cout << "CheckLockState called from line " << lineNo << endl;
  }
}

/* FUNCTION *******************************************************************/
/**
@note PollEventQueue
*******************************************************************************/
static void PollEventQueue( ViSession vi )
{
  ViUInt16 stb = 0;
  
  do 
  {
    int iCode, iCount;
    ViUInt32 size = 0;
    char sResponse[255];

    TR(Send(vi, "SYST:ERR?", true ));
    TR(viRead(vi, (PBYTE)sResponse, sizeof(sResponse) -1, &size) );
    sResponse[size] = '\0';
    iCount = sscanf_s( sResponse, "%d,", &iCode );
    if ( iCount != 1)
    {
      cout << "\nQuery: SYST:ERR? cannont parse '" << sResponse << "'\n";
    }
    else
    {
      if ( -410 != iCode && -113 != iCode )
      {
        cout << "\nQuery: SYST:ERR? returns: '" << sResponse << "'\n";
      }
    }
    TR(viReadSTB( vi, &stb ));
    
  } while( stb & 0x04 );
}

/* FUNCTION *******************************************************************/
/**
@note myrand
*******************************************************************************/
static ULONG32 myrand()
{
  // random between 0 und 2^32.
  return (((((ULONG)rand())<<1) + (rand() & 1)) << 16 ) + ((((ULONG)rand())<<1) + (rand() & 1));
}

/* FUNCTION *******************************************************************/
/**
@note viEventHandler
*******************************************************************************/
static ViStatus _VI_FUNCH viEventHandler(ViSession vi, ViEventType eventType, ViEvent context, ViAddr userHandle )
{
  ViUInt16 stb;
  
  RandomSleep();
  TR(viReadSTB( vi, &stb ));
  
  if ( (stb & 0x40) == 0 )
  {
    if ( s_manfId != 'GK' )  printf("\nWarning: SRQ not set\n");
  }
  
  if ( stb & 0x04 ) s_bSrqEvent = true;
  RandomSleep();
  
  return VI_SUCCESS;
}

/* FUNCTION *******************************************************************/
/**
@note InitSequence
*******************************************************************************/
static void InitSequence( ViSession vi, ViUInt32 uTimeout )
{
  TR(viInstallHandler(vi, VI_EVENT_SERVICE_REQ, viEventHandler, NULL ));
  TR(viEnableEvent( vi, VI_EVENT_SERVICE_REQ, VI_HNDLR, VI_NULL ));
  TR( viSetAttribute( vi, VI_ATTR_TMO_VALUE, uTimeout ) );
  TR(Send( vi, "*CLS;*ESE 0;*SRE 4\n", true ));
}

/* FUNCTION *******************************************************************/
/**
@note StartTest
*******************************************************************************/
void StartTest( ViRsrc name )
{
  bool bIsHiSlip = false;
  bool bIsLocked = false;
  bool bAllowSleep = true;
  ViUInt32 uTimeout = 15000;
  
  cout << "Start test with resource: " << name << endl;
  cout << "Press any key to stop test" << endl;

  char *sFilename = "C:\\test.dat";

  ViSession defRM = NULL;
  ViSession v1 = NULL, v2 = NULL;
  string sLastData;
  unsigned uCount = 0;


  TR( viOpenDefaultRM ( &defRM ) );
  TR( viOpen( defRM, name, VI_NULL, 0, &v1 ) );

  ViUInt32 size;
  if ( VI_SUCCESS == viGetAttribute( v1, VI_ATTR_TCPIP_HISLIP_MAX_MESSAGE_KB, &size ) )
  {
    bIsHiSlip = true;
    TR( viGetAttribute( v1, VI_ATTR_MANF_ID, &s_manfId ));
  }

  if ( VI_SUCCESS != viOpen( defRM, name, VI_NULL, 0, &v2 ) )
  {
    v2 = NULL; // a second session is not possible
  }

  InitSequence( v1, uTimeout );

  for ( ;; ) 
  {
    if ( _kbhit() )
    {
      break;
    }
    
    uCount++;
    cout << "\rCounter: " << uCount;

    if ( s_bSrqEvent )
    {
      TR(viEnableEvent( v1, VI_EVENT_SERVICE_REQ, VI_SUSPEND_HNDLR, VI_NULL ));
      PollEventQueue(v1);
      s_bSrqEvent = false; 
      TR(viEnableEvent( v1, VI_EVENT_SERVICE_REQ, VI_HNDLR, VI_NULL ));
      bAllowSleep = true; // allow case 12 again
    }
    
    int iAction = rand() % 13;

    switch ( iAction )
    {
    case 0:
      TR(viClear( v1 ));
      sLastData.erase();
      break;

    case 1:
      TR(viAssertTrigger( v1, VI_TRIG_PROT_DEFAULT ));
      if ( s_manfId == 'AG' )
      {
        TR(ReadInteger(__LINE__, v1 ));
      }
      break;

    case 2:
      TR(Send( v1, "foo\n", true )); // generate command error 
      break;

    case 3:
      TR(Send( v1, "*STB?\n", true ));
      TR(ReadInteger(__LINE__, v1 ));
      bAllowSleep = true; // allow case 12 again
      break;

    case 4:
      if ( bIsHiSlip )
      {
        TR( viSetAttribute( v1, VI_ATTR_TCPIP_HISLIP_MAX_MESSAGE_KB, (rand()%300)+1 ) );
      }
      break;
      
    case 5:
      if ( s_manfId == 'GK' )
      {
        TR(Send( v1, "DIAG:ARM:SRQ", true )); // generate SRQ with next asynchronous call  
      }
      break;
      
    case 6:
      if ( bIsLocked )
      {
        TR( viUnlock( v1 ) );
      }
      else
      {
        TR( viLock( v1, VI_EXCLUSIVE_LOCK, uTimeout,  NULL, NULL ) );
      }
      bIsLocked = !bIsLocked;
      break;
      
    case 7:
      if ( bIsLocked )
      {
        CheckLockState(  __LINE__  , v1, VI_EXCLUSIVE_LOCK );
      }
      else
      {
        ViAttrState lockState = 0;
        TR( viGetAttribute( v1, VI_ATTR_RSRC_LOCK_STATE, &lockState ));
        if ( lockState != VI_NO_LOCK )
        {
          // R&S server are non-blocking.
          TR(Send( v1, "*OPC?\n", true ));
          TR(ReadInteger(__LINE__, v1 ));
          // But now the server must be unlocked.
          CheckLockState(  __LINE__  , v1, VI_NO_LOCK );
          bAllowSleep = true; // allow case 12 again
        }
      }
      break;
      
#if 1
    case 8:
      TR( viClose(v1) );
      bAllowSleep = true; // allow case 12 again
      s_bSrqEvent = false; // SRQ flag is invalid now..
      if ( rand() % 2 == 1 )
      {
        TR( viOpen( defRM, name, VI_EXCLUSIVE_LOCK, uTimeout, &v1 ) );
        bIsLocked = true;
      }
      else
      {
        TR( viOpen( defRM, name, 0, 0, &v1 ) );
        bIsLocked = false;
      }
        
      InitSequence( v1, uTimeout );
      break;
#endif
      
    case 9:
      {
        //*******************************************************************
        //* Create a test file at server with random bit pattern.
        char sBuffer[250];
        int iLength;
        char sSize[32];

        if ( (rand() % 2) == 0 )
        {
          // long Length
          iLength = (myrand() % 1000000 ) + 1;
        }
        else
        {
          iLength = (rand() % 10) + 1;
        }

        _ultoa_s( iLength, sSize, 10 );
        sprintf_s( sBuffer,":MMEM:DATA '%s',#%d%s", sFilename, strlen(sSize), sSize );
        string sCommand = sBuffer;

        sLastData.erase();
        sLastData.reserve( iLength );

        for ( int i = 0; i < iLength; i++ )
        {
#if 1
          sLastData += char( rand() );
#else
          sLastData += char( (i % 27) + 'a' );
#endif
        }

        sCommand += sLastData;
        if ( (rand() % 2) == 0 )
        {
          // Terminate with NewLine ...
          sCommand +='\n';
        }

        ViUInt32 written;
        ViStatus status = viWrite( v1, (PBYTE)sCommand.data(), static_cast<ViUInt32>(sCommand.size()), &written ); 
        if ( status < VI_SUCCESS )
        {
          CHECK( status == VI_ERROR_TMO );
          sLastData.erase();
        }
        break;
      }

    case 10: 
      {
        //*******************************************************************
        // Read test file and verify the content.
        char sBuffer[250];
        string sRecordData;

        if ( sLastData.size() > 0 )
        {
          bool bError = false;

          // Send Query
          sprintf_s( sBuffer, ":MMEM:DATA? '%s'\n", sFilename );
          TR(Send(v1, sBuffer, true));
          if ( (rand() & 1) == 1 )
          {
            // Some people like to poll status to wait for answer
            CTime startTime = CTime::GetCurrentTime();

            for(;;)
            {
              ViUInt16 stb;
              TR(viReadSTB( v1, &stb ));
              if ( stb & 0x10 ) break;

              Sleep(200); // just for testing
              CTimeSpan span =  CTime::GetCurrentTime() - startTime;
              if ( span.GetSeconds() > 20 )
              {
                cout << "\nWarning: 20 seconds no answer! We continue...\n";
                break;
              }
            }
          }
          
          bAllowSleep = true; // allow case 12 again

          if ( rand() % 4 == 0 )
          {
            // Just read less data to provoke a query error.
            char buf[4096];
            ViUInt32 size;
            ViStatus status = viRead(v1, (PBYTE)buf, (rand() % sizeof(buf)) + 1, &size );
            if ( status < VI_SUCCESS ) TR( status );
          }
          else
          //*************************************************************
          // Read answer and compare bit pattern with last sent pattern.
          {
            CString sError;
            char sHeader[2];
            ViUInt32 uAnswerSize;
            CR(VI_SUCCESS_MAX_CNT, viRead( v1, (PBYTE)sHeader, sizeof(sHeader), &uAnswerSize ));

            if ( uAnswerSize < sizeof(sHeader) )
            {
              sError.Format("response too short!");
              bError = true;
            }
            else if ( sHeader[0] != '#' )
            {
              sError.Format("response does not start with #");
              bError = true;
            }
            else
            {
              int iReadNL = 0;
              char *sAnswer = NULL;
              int iSize = sHeader[1] - '0';
              char sSize[32];
              CR(VI_SUCCESS_MAX_CNT, viRead( v1, (PBYTE)sSize, iSize, &uAnswerSize ));
              sSize[uAnswerSize] = '\0';
              int iCount = 0;
              sscanf_s( sSize, "%d", &iCount );

              if ( iCount != sLastData.size() )
              {
                sError.Format("Wrong size of answer. Expected: %d received: %d", 
                                (sLastData.size()), iCount );
                bError = true;
              }

              if ( !bError )
              {
                uAnswerSize = iCount + 1; 
                sAnswer = new char[uAnswerSize];
                if ( sAnswer != NULL )
                {
                  ViStatus status = viRead(v1, (PBYTE)sAnswer, uAnswerSize, &uAnswerSize );
                  if ( status < VI_SUCCESS ) TR( status );
                }
                else
                {
                  sError.Format("No Memory: Size was %d", uAnswerSize );
                  bError = true;
                }
              }

              if ( !bError && uAnswerSize != (sLastData.size() + 1) )
              {
                sError.Format("Wrong transmission size. Expected: %d received: %d", 
                              (sLastData.size() + 1), uAnswerSize );
                bError = true;
              }

              if ( !bError && memcmp( sLastData.data(), sAnswer, sLastData.size() ) != 0 )
              {
                size_t i;
                const char * p1 = sLastData.data();
                const char * p2 = sAnswer;
                size_t iSize = sLastData.size();

                for ( i = 0; i < iSize; i++, p1++, p2++ )
                {
                  if ( *p1 != *p2 ) break;
                }
                sError.Format("Binary not equal at index: %d (0x%02x != 0x%02x)", 
                              i,(unsigned)*p1,(unsigned)*p2);
                bError = true;
              }

              if (bError && sAnswer != NULL)
              {
                sRecordData.assign(sAnswer,uAnswerSize);
              }
              delete[] sAnswer;
            }

            if ( bError )
            {

              cout << "\nError in Answer: " << sError.GetString() << endl;
              viClose( v1 );
              viClose( v2 );
              viClose( defRM );

#if 0 // Write test data. Use this if you want to compare the files. 
              FILE *fptr = fopen( "Test.dat.sent", "wb" );
              fwrite( sLastData.data(), sLastData.size(), 1, fptr );
              fclose( fptr );
              fptr = fopen( "Test.dat.rec", "wb" );
              fwrite( sRecordData.data(), sRecordData.size(), 1, fptr );
              fclose( fptr );
#endif
              return;
            }
            
          }
        }          
      }

    case 11:
      if ( bIsHiSlip )
      {
        TR(viGpibControlREN( v1, VI_GPIB_REN_ASSERT_ADDRESS ));
      }
      break;

    case 12:
      if ( bAllowSleep )
      {
        SleepOneSecond( v1 );
        bAllowSleep = false; // Sleep is allowed again after a viRead...
      }
      break;
      
    default:
      break;
    }
  }

  TR( viClose( v1 ) );
  TR( viClose( v2 ) );

  TR( viClose( defRM ) );
}

